import {Player, world,system} from "@minecraft/server"
import {ModalFormData,ActionFormData} from '@minecraft/server-ui'

const prefixes = ['!', '=', '-', '+', '=', '÷', '×', '%', '#', '.', '~', '$'];
const forbiddenClans = ['owner','staff','admin', 'null', '0wner', 'own3r', 'niger', 'n-word', 'nword', 'hardr', 'nigger', 'nig', 'n1gg3r', ]

function denyUI(target, nextUI, uiData) {
    target.playSound('error');
    nextUI(target, uiData);
}

function joinClan(viewer,prevError = '') {
    if (!(viewer instanceof Player)) return;

    const ui = new ActionFormData();
    ui.title('Join Clan');
    ui.body('List of clans');

    const clanButtons = new Set(); // Use a Set to store unique clan names.

    for (const player of world.getAllPlayers()) {
        for (const tag of player.getTags()) {
            const parts = tag.split(':');
            if (parts.length === 2) {
                const key = parts[0].trim();
                const value = parts[1].trim();

                if (key === 'Clan') {
                    clanButtons.add(value); // Add the clan name to the Set.
                }
            }
        }
    }

    if (clanButtons.size === 0) {
        ui.body('No clans available.');
    } else {
        clanButtons.forEach((clanName) => {
            ui.button(clanName);
        });
    }
    ui.button('Back')
    ui.show(viewer).then(({ canceled, selection }) => {
        if (canceled) return;
        const selectedClan = [...clanButtons][selection]; // Convert Set to an array and select the clan by index.
        if (selection == clanButtons.length +1){
            search(viewer)
        }
        const existingClanTags = viewer.getTags().filter((tag) => tag.startsWith('Clan:'));
        existingClanTags.forEach((tag) => viewer.removeTag(tag));
        viewer.addTag(`Clan:${selectedClan}`);
        viewer.sendMessage(`You have joined the clan "${selectedClan}"`);
    });
}

function createClan(viewer,prevError = '') {
    if (!(viewer instanceof Player)) return;

    const ui = new ModalFormData();

    // Ask for a clan name
    ui.title("Create Clan");
    ui.textField(`Clan Name §c${prevError}`, "Enter clan name");

    ui.show(viewer).then(({ canceled, formValues }) => {
        if (canceled) return;
        const clanName = formValues[0];

        for (const forbiddenName of forbiddenClans) {
            if (clanName.toLowerCase().includes(forbiddenName.toLowerCase())) {
                denyUI(viewer, createClan, 'Invalid Clan Name');
                return;
            }
        }
        // Check if the clan name contains only allowed characters (alphabet and §).
        const isNameValid = /^[A-Za-z0-9§]+$/.test(clanName);;

        if (!isNameValid) {
            denyUI(viewer,createClan, "Invalid Characters");
            return;
        }

        // Check if the clan name is already taken (assuming players have tags in the format "Clan:clanname").
        const existingClanTag = `Clan:${clanName}`;
        if (viewer.getTags().includes(existingClanTag)) {
            denyUI(viewer,createClan, "Name Taken");
            return;
        }

        // Check if the clan name is too long (greater than 12 characters).
        if (clanName.length > 12) {
            denyUI(viewer,createClan, "Name Too Long");
            return;
        }

        // Remove all existing clan tags and give the selected one
        const existingClanTags = viewer.getTags().filter((tag) => tag.startsWith('Clan:'));
        existingClanTags.forEach((tag) => viewer.removeTag(tag));
        viewer.addTag(existingClanTag);

        viewer.sendMessage(`You have successfully created the clan "${clanName}"`);
    });
}


// Main menu
function search(viewer,prevError = '') {
    const ui = new ActionFormData();
    ui.title("Clan Menu");

    ui.button('Create');
    ui.button('Join');

    ui.show(viewer).then(({canceled,selection})=>{
        if (canceled)return
        if (selection === 0){
            createClan(viewer)
        }
        if (selection ===1){
            joinClan(viewer)
        }
    });
}

world.beforeEvents.chatSend.subscribe((data) => {
    const player = data.sender;
    const message = data.message;
    const command = data.message.split(' ')
    const tags = player.getTags();
    switch (command[0]){
        case "%clans":
            data.cancel = true
            player.sendMessage("§cClose chat to open menu")
            system.runTimeout(()=>{search(player)},80)
            return
        default:break
    }



    for (const prefix of prefixes) {
        if (message.startsWith(prefix)) {
            return;
        }
    }

    let clan = null;
    let rank = null;

    for (const tag of tags) {
        const parts = tag.split(':');
        if (parts.length === 2) {
            const key = parts[0].trim();
            const value = parts[1].trim();

            if (key === 'Clan') {
                clan = value;
            } else if (key === 'Rank') {
                rank = value;
            }
        }
    }
    if (clan != null && rank != null){
        data.cancel = true
        world.sendMessage(`§g[§r${rank}§r§g]§r §8[§r${clan}§r§8]§r <${player.name}§r> ${message}`)
        return
    }
    if (clan != null){
        data.cancel = true
        world.sendMessage(`§g[§r§bMember§r§g]§r §8[§r${clan}§r§8]§r <${player.name}§r> ${message}`)
        return
    }
    if (rank != null){
        data.cancel = true
        world.sendMessage(`§g[§r${rank}§r§g]§r <${player.name}§r> ${message}`)
        return
    }
});